"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/** @jest-environment puppeteer */
/* eslint-env jasmine */
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
function sleep(ms) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve) => setTimeout(resolve, ms));
    });
}
function getAttrs() {
    return __awaiter(this, void 0, void 0, function* () {
        return page.$eval('.ps-table', (el) => {
            return {
                count: el.getAttribute('data-ps-last-input-processed'),
                isAcceptingInput: el.getAttribute('data-ps-accepting-input'),
                levelNum: el.getAttribute('data-ps-current-level')
            };
        });
    });
}
function pressKeys(keys) {
    return __awaiter(this, void 0, void 0, function* () {
        for (const key of keys) {
            if (key === ',') {
                continue;
            }
            yield page.waitFor(`.ps-table[data-ps-accepting-input='true']`);
            if (key === '.') {
                // wait long enough for a tick to occur
                yield sleep(100);
                continue;
            }
            const { count } = yield getAttrs();
            // await page.keyboard.press(`Key${key}`)
            yield page.keyboard.down(`Key${key}`);
            yield sleep(100); // because alerts might show up and they take some time to pop open?
            yield page.keyboard.up(`Key${key}`);
            yield sleep(100);
            // wait until the keypress was processed
            yield page.waitFor(`.ps-table:not([data-ps-last-input-processed='${count}'])`);
        }
    });
}
// redirect browser console messages to the terminal
const consoleHandler = (message) => {
    const type = message.type();
    const text = message.text();
    switch (type) {
        case 'log':
            console.log(text);
            break; // tslint:disable-line:no-console
        case 'debug':
            console.debug(text);
            break; // tslint:disable-line:no-console
        case 'info':
            console.info(text);
            break; // tslint:disable-line:no-console
        case 'error':
            console.error(text);
            break; // tslint:disable-line:no-console
        case 'dir':
            console.dir(text);
            break; // tslint:disable-line:no-console
        case 'dirxml':
            console.dirxml(text);
            break; // tslint:disable-line:no-console
        case 'table':
            console.table(text);
            break; // tslint:disable-line:no-console
        case 'trace':
            console.trace(text);
            break; // tslint:disable-line:no-console
        case 'assert':
            console.assert(text);
            break; // tslint:disable-line:no-console
        case 'profile':
            console.profile(text);
            break; // tslint:disable-line:no-console
        case 'profileEnd':
            console.profileEnd(text);
            break; // tslint:disable-line:no-console
        case 'count':
            console.count(text);
            break; // tslint:disable-line:no-console
        case 'timeEnd':
            console.timeEnd(text);
            break; // tslint:disable-line:no-console
        case 'clear':
            console.clear();
            break; // tslint:disable-line:no-console
        case 'warning':
            console.warn(text);
            break; // tslint:disable-line:no-console
        case 'startGroup':
        case 'startGroupCollapsed':
        case 'endGroup':
            console.info(type, text);
            break; // tslint:disable-line:no-console
    }
};
function evaluateWithStackTrace(fn, ...args) {
    return __awaiter(this, void 0, void 0, function* () {
        // try {
        return page.evaluate(fn, ...args);
        // } catch (e) {
        //     const stack = e.stack
        //     const message = stack.split('\n')[0]
        //     const newStack = await mapStackTrace(stack, { isChromeOrEdge: true })
        //     console.error(`${message}\n${newStack}`)
        //     e.stack = newStack
        //     throw e
        // }
    });
}
function playLevel() {
    return __awaiter(this, void 0, void 0, function* () {
        const source = fs_1.default.readFileSync(path_1.default.join(__dirname, '../../gists/_pot-wash-panic_itch/script.txt'), 'utf-8');
        const startLevel = 3;
        // This variable is _actually_ defined in the JS file, not here but it is in the body of page.evaluate
        const HackTableStart = (sourceBrowser, startLevelBrowser) => 'actually implemented in the browser';
        yield evaluateWithStackTrace(({ source, startLevel }) => {
            if (HackTableStart) {
                if (source && typeof startLevel === 'number') {
                    HackTableStart(source, startLevel);
                }
                else {
                    throw new Error(`BUG: Source was not a string or startLevel was not a number`);
                }
            }
            else {
                throw new Error(`BUG: The browser JS does not have HackTableStart defined`);
            }
        }, { source, startLevel });
        yield pressKeys('SAAASASDDDWDDDDWDDSAAASASAW'.split(''));
    });
}
describe('Browser', () => {
    const dismissedCount = [];
    const dialogHandler = (dialog) => __awaiter(this, void 0, void 0, function* () {
        dismissedCount.push(dialog.message());
        yield dialog.dismiss();
    });
    beforeEach(() => {
        // jest-puppeteer will expose the `page` and `browser` globals to Jest tests.
        if (!browser || !page) {
            throw new Error('Browser has not been started! Did you remember to specify `@jest-environment puppeteer`?');
        }
        page.on('console', consoleHandler);
        page.on('dialog', dialogHandler);
        // page.on('pageerror', async e => {
        //     const newStack = await mapStackTrace(e.message, { isChromeOrEdge: true })
        //     console.error(newStack)
        // })
    });
    afterEach(() => {
        // Node 8 does not have EventListener.off(...)
        page.removeListener('console', consoleHandler);
        page.removeListener('dialog', dialogHandler);
    });
    it('plays a game in the browser using the SyncTableEngine', () => __awaiter(this, void 0, void 0, function* () {
        // browser tests are slow. Headless is slower it seems (from jest watch mode)
        jest.setTimeout(process.env.NODE_ENV === 'development' ? 90 * 1000 : 90 * 1000);
        yield page.goto(`http://localhost:8000/src/browser/test/html-table.xhtml`);
        return playLevel();
    }));
    it('plays a game in the browser using the WebworkerTableEngine', () => __awaiter(this, void 0, void 0, function* () {
        // browser tests are slow. Headless is slower it seems (from jest watch mode)
        jest.setTimeout(process.env.NODE_ENV === 'development' ? 90 * 1000 : 90 * 1000);
        yield page.goto(`http://localhost:8000/src/browser/test/html-table-webworker.xhtml`);
        return playLevel();
    }));
    it('plays a couple levels using the demo page', () => __awaiter(this, void 0, void 0, function* () {
        const waitForDialogAfter = (fn) => __awaiter(this, void 0, void 0, function* () {
            // page.once('dialog', dialogHandler)
            const oldCount = dismissedCount.length;
            yield fn();
            // Keep checking for the dialog to be dismissed
            if (dismissedCount.length > oldCount) {
                return;
            }
            yield sleep(100); // wait a little bit
            if (dismissedCount.length > oldCount) {
                return;
            }
            yield sleep(1000); // wait for the dialog to open and be dismissed
            if (dismissedCount.length > oldCount) {
                return;
            }
            yield sleep(10000); // wait a long time
            expect(dismissedCount.length).toBeGreaterThan(oldCount);
        });
        // The game shows a dialog immediately (uggh)
        yield waitForDialogAfter(() => __awaiter(this, void 0, void 0, function* () {
            yield page.goto(`http://localhost:8000/index.xhtml`);
            yield page.waitForSelector(`#loadingIndicator.hidden`);
        }));
        // play a level and then wait for the dialog to open
        yield page.waitForSelector(`.ps-table[data-ps-current-level="${1}"]`);
        yield waitForDialogAfter(() => __awaiter(this, void 0, void 0, function* () { return pressKeys('.AWAW'.split('')); }));
        yield page.waitForSelector(`.ps-table[data-ps-current-level="${3}"]`);
        yield waitForDialogAfter(() => __awaiter(this, void 0, void 0, function* () { return pressKeys('.WASDW'.split('')); }));
        yield page.waitForSelector(`.ps-table[data-ps-current-level="${5}"]`);
        expect(dismissedCount.length).toBe(5);
    }));
});
//# sourceMappingURL=browser.test.js.map