"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("../util");
class ResizeWatcher {
    constructor(table, handler) {
        this.table = table;
        this.handler = handler;
        this.columns = 1;
        this.rows = 1;
        this.boundResizeHandler = util_1._debounce(this.trigger.bind(this));
        window.addEventListener('resize', this.boundResizeHandler);
    }
    setLevel(rows, columns) {
        this.rows = rows;
        this.columns = columns;
    }
    trigger() {
        // Resize the table so that it fits.
        const levelRatio = this.columns / this.rows;
        // Figure out if the width or the height is the limiting factor
        const availableWidth = window.innerWidth - this.table.offsetLeft;
        const availableHeight = window.innerHeight - this.table.offsetTop;
        let newWidth = 0;
        if (availableWidth / availableHeight < levelRatio) {
            // Width is the limiting factor
            newWidth = availableWidth / levelRatio;
        }
        else {
            // Height is the limiting factor
            newWidth = availableHeight * levelRatio;
        }
        this.handler(Math.floor(newWidth));
    }
    dispose() {
        window.removeEventListener('resize', this.boundResizeHandler);
    }
}
exports.default = ResizeWatcher;
//# sourceMappingURL=ResizeWatcher.js.map